/*** Sample Program Description ************************************


Sample Name:		RCH_DEMO1.C

Description:		Sample program using the REACH TECH. SLCD
						Controller with and Hitachi TFT Display, connected
						to the BL2500.
						
Instructions:		Load the bitmaps into the display using
						the bmploader program supplied by Reach Tech.
						Make sure you use the ZWORLD.LST file attached.
						
						Run the Sample.
						




END DESCRIPTION ***************************************************/


#class auto
#memmap xmem


//#define	RCH_DEBUG                   // Allows for single stepping
#define	RCH_CALIBRATE_ON_START	    // runs the ts calibration at startup
//#define	RCH_DEBUG_COMM              // displays the rx/tx msgs in STDIO



// Uncomment 1 of the serial port macros below, default for the BL2500
// is RCH_USE_SER_E

//#define	RCH_USE_SER_A
//#define	RCH_USE_SER_B
//#define	RCH_USE_SER_C
//#define	RCH_USE_SER_D

#define	RCH_USE_SER_E
//#define	RCH_USE_SER_F

// Setup the Serial port buffer sizes
#define	RCH_INBUFSIZE			255
#define	RCH_OUTBUFSIZE			255

// Macros for the input switch of the demo board.
#define S1	0    // DigIn value
#define S2  1		
#define S3  2		
#define S4  3
// Macros for the LEDS of the demo board

#define DS1 0		// digOut value.		
#define DS2 1		
#define DS3 2
#define DS4 3

// Setup baudrate, communication timeout, and serial port macro.
#define	RCH_BAUDRATE			115200
#define	RCH_COMM_TMO			100L
#define	RCH_MY_SERPORT			RCH_SER_E

// Bitmap id numbers.  These must match in the order that was download
// to the SLCD controller via the bmploader utility.

#define	BMP_REG    			2
#define	BMP_INPUT_ON    	3
#define	BMP_OUTPUT_OFF		10
#define	BMP_OUTPUT_ON     11
#define	BMP_BTN				7
#define	BMP_BTN_CLICK		8
#define	BMP_LOGO				9
#define  BMP_KEYBOARD		12

// library specific for the reach SLCD Controller
#use	"reach_tech.lib"

// Macros for screen numbers, btn iD numbers, etc.
#define	SCRN_MAIN	0
#define	SCRN_DEMO	1
#define	SCRN_SETUP	2
#define	SCRN_BRIGHTNESS 3
#define	SCRN_TIMEDATE	4

#define	BTN_DEMO		0
#define	BTN_SETUP	1
#define	BTN_EXIT		2
#define	BTN_IN1		3
#define	BTN_IN2		4
#define	BTN_IN3		5
#define	BTN_IN4		6

#define	BTN_OUT1		180
#define	BTN_OUT2		181
#define	BTN_OUT3		182
#define	BTN_OUT4		183

#define	BTN_KEY_RET 175

#define	BTN_SET_BRIGHTNESS	12
#define	BTN_SET_TIME		13
#define	BTN_SET_DATE		14
#define	BTN_CALIBRATE		15

#define	BTN_BRIGHTNESS_UP	16
#define	BTN_BRIGHTNESS_DN	17
#define	BTN_EXIT_BRIGHTNESS	18
#define	BTN_EXIT_TIMEDATE		19


// display control structure
RCH_STRUCT	display1;

// misc global variables
char btnOutState[4];
char digInState;
char digInLastState;

// Main Screen display function
nodebug
int	rch_MainScrn(RCH_STRUCT *display)
{
	static int retval;
 	costate 
 	{
 		retval = 0;
 		waitfor(DelayMs(350));
 		waitfor(RCH_ClearScreen(display));
 		waitfor(RCH_DispBitmap(display, BMP_LOGO,0, 0));
 		waitfor(DelayMs(100));
 		waitfor(RCH_ButtonDefine(display,BTN_DEMO,5,207,BMP_BTN,BMP_BTN_CLICK,RCH_BTNTYPE_REGULAR,
 	 	                         "DEMO",20,5,RCH_BTNDEFINE_END));
 	 	waitfor(RCH_ButtonDefine(display,BTN_SETUP,235,207,BMP_BTN,BMP_BTN_CLICK,RCH_BTNTYPE_REGULAR,
 	 	                         "SETUP",15,5,RCH_BTNDEFINE_END));
      retval = 1;
 	}
   return retval;
}

// Demo Screen display function
nodebug
int	rch_DemoScrn(RCH_STRUCT *display)
{
	static int retval;
 	costate 
 	{
 		retval = 0;
 		waitfor(RCH_ClearScreen(display));
 		waitfor(RCH_SetColor(display,RCH_COLOR_BLUE,RCH_COLOR_WHITE));
 		waitfor(RCH_SetFont(display,RCH_FONT_32B));
 		waitfor(RCH_DispText(display,"I/O BOARD DEMO",160 - ((strlen("I/O BOARD DEMO") * 16) /2),10));
 		waitfor(RCH_SetColor(display,RCH_COLOR_BLACK,RCH_COLOR_WHITE));
 		waitfor(RCH_SetFont(display,RCH_FONT_16B));
 		waitfor(RCH_ButtonDefine(display,BTN_EXIT,235,207,BMP_BTN,BMP_BTN_CLICK,RCH_BTNTYPE_REGULAR,
 	 	                         "EXIT",20,5,RCH_BTNDEFINE_END));
 		waitfor(RCH_DispBitmap(display,BMP_REG,10,150));
 		waitfor(RCH_DispBitmap(display,BMP_REG,85,150));
 		waitfor(RCH_DispBitmap(display,BMP_REG,165,150));
 		waitfor(RCH_DispBitmap(display,BMP_REG,245,150));
 		waitfor(RCH_DispText(display,"IN 1",25,130));
 		waitfor(RCH_DispText(display,"IN 2",105,130));
 	 	waitfor(RCH_DispText(display,"IN 3",185,130));
 	 	waitfor(RCH_DispText(display,"IN 4",265,130));
 		waitfor(RCH_DispText(display,"OUT 1",22,50));
 		waitfor(RCH_DispText(display,"OUT 2",100,50));
 	 	waitfor(RCH_DispText(display,"OUT 3",177,50));
 	 	waitfor(RCH_DispText(display,"OUT 4",254,50));
 	 	waitfor(RCH_DefineTouchArea(display,BTN_OUT1,RCH_TOUCH_HIDDEN,10,65,63,135));
 	 	waitfor(RCH_DefineTouchArea(display,BTN_OUT2,RCH_TOUCH_HIDDEN,85,65,143,135));
 	 	waitfor(RCH_DefineTouchArea(display,BTN_OUT3,RCH_TOUCH_HIDDEN,165,65,213,135));
 	 	waitfor(RCH_DefineTouchArea(display,BTN_OUT4,RCH_TOUCH_HIDDEN,245,65,303,135));
 	 	waitfor(RCH_DispBitmap(display,btnOutState[0] + BMP_OUTPUT_OFF,10,65));
 		waitfor(RCH_DispBitmap(display,btnOutState[1] + BMP_OUTPUT_OFF,85,65));
 		waitfor(RCH_DispBitmap(display,btnOutState[2] + BMP_OUTPUT_OFF,165,65));
 		waitfor(RCH_DispBitmap(display,btnOutState[3] + BMP_OUTPUT_OFF,245,65));
 	 	
 	 	
 		
      retval = 1;
 	}
   return retval;
}
// Setup screen display function
nodebug
int	rch_SetupScrn(RCH_STRUCT *display)
{
	static int retval;
 	costate 
 	{
 		retval = 0;
 		waitfor(RCH_ClearScreen(display));
 		waitfor(RCH_SetColor(display,RCH_COLOR_BLUE,RCH_COLOR_WHITE));
 		waitfor(RCH_SetFont(display,RCH_FONT_32B));
 		waitfor(RCH_DispText(display,"SETUP SCREEN",140 - ((strlen("SETUP SCREEN") * 16) /2),10));
 		waitfor(RCH_SetColor(display,RCH_COLOR_BLACK,RCH_COLOR_WHITE));
 		waitfor(RCH_SetFont(display,RCH_FONT_16B));
 		
 		waitfor(RCH_ButtonDefine(display,BTN_EXIT,235,207,BMP_BTN,BMP_BTN_CLICK,RCH_BTNTYPE_REGULAR,
 	 	                         "EXIT",20,5,RCH_BTNDEFINE_END));
 	 	                         
 	 	waitfor(RCH_ButtonDefine(display,BTN_CALIBRATE,5,60,BMP_BTN,BMP_BTN_CLICK,RCH_BTNTYPE_REGULAR,
 	 	                         "",0,0,RCH_BTNDEFINE_END));
 	 	waitfor(RCH_DispText(display,"Calibrate Touch Screen",92,68));
 	 	
 		waitfor(RCH_ButtonDefine(display,BTN_SET_BRIGHTNESS,5,95,BMP_BTN,BMP_BTN_CLICK,RCH_BTNTYPE_REGULAR,
 	 	                         "",0,0,RCH_BTNDEFINE_END));
 	 	waitfor(RCH_DispText(display,"Set Brightness",92,103));
 	 	
 		waitfor(RCH_ButtonDefine(display,BTN_SET_TIME,5,130,BMP_BTN,BMP_BTN_CLICK,RCH_BTNTYPE_REGULAR,
 	 	                         "",0,0,RCH_BTNDEFINE_END));
 	 	waitfor(RCH_DispText(display,"Virtual Keyboard Demo",92,138));
 	 	
 	 	waitfor(RCH_SetColor(display,RCH_COLOR_GREEN,RCH_COLOR_WHITE));
 		waitfor(RCH_DrawRectangle(display,1,1,317,237,0,""));
 		waitfor(RCH_SetColor(display,RCH_COLOR_BLACK,RCH_COLOR_WHITE));
 		
      retval = 1;
 	}
   return retval;
}
// Btn retrieval function
nodebug
int	rch_GetTouch(RCH_STRUCT *display, int *Touchnumber)
{
 	auto char BtnStr[6];
 	if (RCH_GetPkt (&display1,BtnStr,sizeof(BtnStr)))
 	{
 	 	if (BtnStr[0] == 'x')
 	 	{ 
 	 		*Touchnumber = atoi(&BtnStr[1]);	
 	   	return 1;
 	   } 
 	} 
   return 0;
}

// brightness control function
nodebug
int	rch_BrightnessScrn(RCH_STRUCT *display)
{
	static int retval;
	costate
 	{
 		retval = 0;
 	  	waitfor(RCH_ClearScreen(display));
 		waitfor(RCH_SetColor(display,RCH_COLOR_BLUE,RCH_COLOR_WHITE));
 		waitfor(RCH_SetFont(display,RCH_FONT_32B));
 		waitfor(RCH_DispText(display,"BRIGHTNESS SCREEN",140 - ((strlen("BRIGHTNESS SCREEN") * 16) /2),10));
 		waitfor(RCH_SetColor(display,RCH_COLOR_BLACK,RCH_COLOR_WHITE));
 		waitfor(RCH_SetFont(display,RCH_FONT_16B));	
 	   waitfor(RCH_ButtonDefine(display,BTN_BRIGHTNESS_UP,5,75,BMP_BTN,BMP_BTN_CLICK,RCH_BTNTYPE_REGULAR, 
 	                         "",0,0,RCH_BTNDEFINE_END));
 		waitfor(RCH_DispText(display,"Increase Brightness",92,83));
 		
 		waitfor(RCH_ButtonDefine(display,BTN_BRIGHTNESS_DN,5,110,BMP_BTN,BMP_BTN_CLICK,RCH_BTNTYPE_REGULAR,
 	 	                         "",0,0,RCH_BTNDEFINE_END));
 		waitfor(RCH_DispText(display,"Decrease Brightness",92,118));
 		waitfor(RCH_ButtonDefine(display,BTN_EXIT_BRIGHTNESS,235,207,BMP_BTN,BMP_BTN_CLICK,RCH_BTNTYPE_REGULAR,
 	 	                         "EXIT",20,5,RCH_BTNDEFINE_END));
 	 	waitfor(RCH_SetColor(display,RCH_COLOR_GREEN,RCH_COLOR_WHITE));
 		waitfor(RCH_DrawRectangle(display,1,1,317,237,0,""));
 		waitfor(RCH_SetColor(display,RCH_COLOR_BLACK,RCH_COLOR_WHITE));
 	 	retval =  1;
 	}
   return retval;

}

// Main function
main ()
{
 	static char Pkt[100];
 	static int touchVal;
 	static char CurScrn;
 	static char dispStr[100];
 	memset(btnOutState,0x00,sizeof(btnOutState));
 	brdInit();
 	RCH_Open(&display1, RCH_MY_SERPORT,RCH_BAUDRATE,RCH_COMM_TMO);
 	while (!RCH_Connect (&display1));  	
   while (!RCH_ClearScreen(&display1));
#ifdef RCH_CALIBRATE_ON_START
	while (!RCH_ResetTouchCal (&display1));
   while (!RCH_TouchCalibrate(&display1));
#endif
	while (!RCH_SetTypematic (&display1,50,25));  
														
   CurScrn = SCRN_MAIN; 
   digInState = 0;
   digInState = 0xff;
   while (!rch_MainScrn(&display1));
   for (;;)
   {
   	costate
   	{
   	 	waitfor(rch_GetTouch(&display1, &touchVal));
   	 	// act on the touch screen ID number that was pressed.
   	 	switch (touchVal)  
   	 	{
   	 	 	case BTN_DEMO:
   	 	 		waitfor(rch_DemoScrn(&display1));
   	 	 		digInLastState = 0x00;
   	 	 		CurScrn = SCRN_DEMO;
   	 	 		break;
   	 	 	case BTN_SETUP:
   	 	 		waitfor(rch_SetupScrn(&display1));
   	 	 		CurScrn = SCRN_SETUP;
   	 	 		break;
   	 		case BTN_EXIT:
   	 			waitfor(rch_MainScrn(&display1));
   	 			CurScrn = SCRN_MAIN;
   	 			break;
   	 		case BTN_OUT1:
   	 			if (btnOutState[0]) btnOutState[0] = 0;
   	 			else btnOutState[0] = 1;
   	 			digOut(DS1,!btnOutState[0]);       
					
					waitfor(RCH_DispBitmap(&display1,(btnOutState[0]  * 1)+ BMP_OUTPUT_OFF,10,65));					       
   	 			break;
   	 		case BTN_OUT2:    
   	 			if (btnOutState[1]) btnOutState[1] = 0;
   	 			else btnOutState[1] = 1;
   	 			digOut(DS2,!btnOutState[1]);
   	 			waitfor(RCH_DispBitmap(&display1,(btnOutState[1] * 1) + BMP_OUTPUT_OFF,85,65));   	 			        
   	 			break;
   	 		case BTN_OUT3:    
   	 			if (btnOutState[2]) btnOutState[2] = 0;
   	 			else btnOutState[2] = 1;
   	 			digOut(DS3,!btnOutState[2]);
   	 			waitfor(RCH_DispBitmap(&display1,(btnOutState[2] * 1) + BMP_OUTPUT_OFF,165,65));  	 			        
   	 			break;
   	 		case BTN_OUT4:    
   	 			if (btnOutState[3]) btnOutState[3] = 0;
   	 			else btnOutState[3] = 1;
   	 			digOut(DS4,!btnOutState[3]);
   	 			waitfor(RCH_DispBitmap(&display1,(btnOutState[3] * 1) + BMP_OUTPUT_OFF,245,65));   	 			        
   	 			break;
   	 		case BTN_CALIBRATE:
   	 			waitfor(DelayMs(1000));
   	 			waitfor(RCH_TouchCalibrate (&display1));
   	 			waitfor(rch_SetupScrn(&display1));        
   	 			break;
   	 		case BTN_SET_BRIGHTNESS:
   	 			waitfor(rch_BrightnessScrn(&display1));
   	 			CurScrn = SCRN_BRIGHTNESS;
   	 			break;
   	 		case BTN_EXIT_BRIGHTNESS:
   	 			waitfor(rch_SetupScrn(&display1));
   	 			CurScrn = SCRN_SETUP;
   	 			break;     
   	   	case BTN_BRIGHTNESS_UP:
   	   		waitfor(RCH_Brightness (&display1,"+50"));
   	   		break;
   	   	case BTN_BRIGHTNESS_DN:
   	   		waitfor(RCH_Brightness (&display1,"-50")); 
   	   		break;
   	   	case BTN_SET_TIME:
   	   		CurScrn = SCRN_TIMEDATE;
   	   		waitfor(RCH_EnterString(&display1,dispStr,sizeof(dispStr), "Enter A New String Value"));
   	   		printf("\n%s\n",dispStr);
   	   		waitfor(rch_SetupScrn(&display1));
   	 			CurScrn = SCRN_SETUP;
   	 			
   	   		break;
   	   	case BTN_EXIT_TIMEDATE:
   	   		waitfor(rch_SetupScrn(&display1));
   	 			CurScrn = 2;
   	   		break;
   	   	case BTN_KEY_RET:
   	   		waitfor(rch_SetupScrn(&display1));
   	 			CurScrn = 2;
   	   		break;    
   		}
   	}
   	// controls the display for the demo board sample.
   	costate
   	{
   	 	waitfor(CurScrn == SCRN_DEMO);
   	 	if (digIn(0)) res(&digInState,0);
   	 	else	set(&digInState,0);    
   	 	if (digIn(1)) res(&digInState,1);
   	 	else	set(&digInState,1);
   	 	if (digIn(2)) res(&digInState,2);
   	 	else	set(&digInState,2);
   	 	if (digIn(3)) res(&digInState,3);
   	 	else	set(&digInState,3);
   	 	if (digInLastState != digInState)
   	 	{
   	 	 	if (bit(&digInState,S1)) 
   	 	 			waitfor(RCH_DispBitmap(&display1,BMP_REG,10,150));    
   	 		else 	waitfor(RCH_DispBitmap(&display1,BMP_INPUT_ON,10,150));
   	 		     
   	 		if (bit(&digInState,S2)) 
   	 				waitfor(RCH_DispBitmap(&display1,BMP_REG,85,150));     
   	 		else 	waitfor(RCH_DispBitmap(&display1,BMP_INPUT_ON,85,150));
   	 		
   	 		if (bit(&digInState,S3)) 
   	 				waitfor(RCH_DispBitmap(&display1,BMP_REG,165,150));     
   	 		else 	waitfor(RCH_DispBitmap(&display1,BMP_INPUT_ON,165,150));
   	 		
   	 		if (bit(&digInState,S4)) 
   	 				waitfor(RCH_DispBitmap(&display1,BMP_REG,245,150));     
   	 		else 	waitfor(RCH_DispBitmap(&display1,BMP_INPUT_ON,245,150));
   	 		 
   	 		digInLastState = digInState; 
   	 		waitfor(DelayMs(50));   
   	 	}  	 
   	}
   	
	}
}